package speiger.src.builder.conditions;

import java.util.Arrays;
import java.util.Map;
import java.util.StringJoiner;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import speiger.src.builder.processor.CompiledArguments;

public class NumberCondition implements ICondition
{
	private static final Pattern SCANNER = Pattern.compile(Operation.buildReggex());
	String flag;
	Operation op;
	int number;
	boolean flipped;
	
	public NumberCondition(String flag, Operation op, int number, boolean flipped)
	{
		this.flag = flag;
		this.op = op;
		this.number = number;
		this.flipped = flipped;
	}
	
	@Override
	public boolean isValid(CompiledArguments args)
	{
		return op.matches(flipped ? args.get(flag) : number, flipped ? number : args.get(flag));
	}
	
	public static NumberCondition tryBuild(String arg)
	{
		Matcher match = SCANNER.matcher(arg);
		if(!match.find()) return null;
		int start = match.start();
		int end = match.end();
		Operation op = Operation.byId(arg.substring(start, end));
		if(op == null) throw new IllegalArgumentException("Operation ["+arg.substring(start, end)+"] is unknown");
		String key = arg.substring(0, start);
		String value = arg.substring(end);
		int[] result = new int[1];
		if(parse(key, result)) return new NumberCondition(value, op, result[0], true);
		if(parse(value, result)) return new NumberCondition(key, op, result[0], false);
		throw new IllegalStateException("Neither key or value are parsable");
	}
	
	
	private static boolean parse(String arg, int[] output) {
		try { output[0] = Integer.parseInt(arg); }
		catch(Exception e) { return false;}
		return true;
	}
	
	public static enum Operation
	{
		EQUALS("=="),
		GREATER_EQUALS(">="),
		GREATER(">"),
		SMALLER_EQUALS("<="),
		SMALLER("<"),
		NOT_EQUALS("!="),
		BIT_EQUALS_ALL("&="),
		BIT_EQUALS_ANY("|="),
		BIT_EQUALS_NOT("~=");
		
		static final Map<String, Operation> DATA = Arrays.stream(values()).collect(Collectors.toMap(Operation::getId, Function.identity()));
		String key;

		private Operation(String key)
		{
			this.key = key;
		}
		
		public static String buildReggex()
		{
			StringJoiner joiner = new StringJoiner("|\\", "\\", "");
			for(Operation entry : values())
			{
				joiner.add(entry.getId());
			}
			return joiner.toString();
		}
		
		public String getId()
		{
			return key;
		}
		
		public boolean matches(int key, int value)
		{
			switch(this)
			{
				case EQUALS: return key == value;
				case NOT_EQUALS: return key != value;
				case GREATER: return key > value;
				case GREATER_EQUALS: return key >= value;
				case SMALLER: return key < value;
				case SMALLER_EQUALS: return key <= value;
				case BIT_EQUALS_ALL: return (key & value) == value;
				case BIT_EQUALS_ANY: return (key & value) != 0;
				case BIT_EQUALS_NOT: return (key & value) == 0;
				default: return true;
			}
		}
		
		public static Operation byId(String key)
		{
			return DATA.get(key);
		}
	}
}